`timescale 1 ps / 1 ps

module hdmi_tx_top(
    input        clk_100,
    input        rst,
    
    output       clk_pix,
    output       clk_locked,
    input        de,
    input        hs,
    input        vs,
    input  [7:0] data_in_b,
    input  [7:0] data_in_g,
    input  [7:0] data_in_r,
    
    output       clk_o_p,
    output       clk_o_n,
    output [2:0] data_o_p,
    output [2:0] data_o_n
);

wire mmcm_clk_5x;
wire mmcm_clk_1x;
wire mmcm_clkfb;
wire clk_5x;
wire clk_1x;
wire mmcm_locked;
MMCME2_BASE #(
  .BANDWIDTH("OPTIMIZED"),   // Jitter programming (OPTIMIZED, HIGH, LOW)
  .CLKFBOUT_MULT_F(54.0),     // Multiply value for all CLKOUT (2.000-64.000).
  .CLKFBOUT_PHASE(0.0),      // Phase offset in degrees of CLKFB (-360.000-360.000).
  .CLKIN1_PERIOD(10.0),       // Input clock period in ns to ps resolution (i.e. 33.333 is 30 MHz).
  // CLKOUT0_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
  .CLKOUT1_DIVIDE(10),
  .CLKOUT2_DIVIDE(1),
  .CLKOUT3_DIVIDE(1),
  .CLKOUT4_DIVIDE(1),
  .CLKOUT5_DIVIDE(1),
  .CLKOUT6_DIVIDE(1),
  .CLKOUT0_DIVIDE_F(2.0),    // Divide amount for CLKOUT0 (1.000-128.000).
  // CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.01-0.99).
  .CLKOUT0_DUTY_CYCLE(0.5),
  .CLKOUT1_DUTY_CYCLE(0.5),
  .CLKOUT2_DUTY_CYCLE(0.5),
  .CLKOUT3_DUTY_CYCLE(0.5),
  .CLKOUT4_DUTY_CYCLE(0.5),
  .CLKOUT5_DUTY_CYCLE(0.5),
  .CLKOUT6_DUTY_CYCLE(0.5),
  // CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
  .CLKOUT0_PHASE(0.0),
  .CLKOUT1_PHASE(0.0),
  .CLKOUT2_PHASE(0.0),
  .CLKOUT3_PHASE(0.0),
  .CLKOUT4_PHASE(0.0),
  .CLKOUT5_PHASE(0.0),
  .CLKOUT6_PHASE(0.0),
  .CLKOUT4_CASCADE("FALSE"), // Cascade CLKOUT4 counter with CLKOUT6 (FALSE, TRUE)
  .DIVCLK_DIVIDE(5),         // Master division value (1-106)
  .REF_JITTER1(0.0),         // Reference input jitter in UI (0.000-0.999).
  .STARTUP_WAIT("FALSE")     // Delays DONE until MMCM is locked (FALSE, TRUE)
)
MMCME2_BASE_inst (
  // Clock Outputs: 1-bit (each) output: User configurable clock outputs
  .CLKOUT0(mmcm_clk_5x),     // 1-bit output: CLKOUT0
  .CLKOUT0B(),   // 1-bit output: Inverted CLKOUT0
  .CLKOUT1(mmcm_clk_1x),     // 1-bit output: CLKOUT1
  .CLKOUT1B(),   // 1-bit output: Inverted CLKOUT1
  .CLKOUT2(),     // 1-bit output: CLKOUT2
  .CLKOUT2B(),   // 1-bit output: Inverted CLKOUT2
  .CLKOUT3(),     // 1-bit output: CLKOUT3
  .CLKOUT3B(),   // 1-bit output: Inverted CLKOUT3
  .CLKOUT4(),     // 1-bit output: CLKOUT4
  .CLKOUT5(),     // 1-bit output: CLKOUT5
  .CLKOUT6(),     // 1-bit output: CLKOUT6
  // Feedback Clocks: 1-bit (each) output: Clock feedback ports
  .CLKFBOUT(mmcm_clkfb),   // 1-bit output: Feedback clock
  .CLKFBOUTB(), // 1-bit output: Inverted CLKFBOUT
  // Status Ports: 1-bit (each) output: MMCM status ports
  .LOCKED(mmcm_locked),       // 1-bit output: LOCK
  // Clock Inputs: 1-bit (each) input: Clock input
  .CLKIN1(clk_100),       // 1-bit input: Clock
  // Control Ports: 1-bit (each) input: MMCM control ports
  .PWRDWN(1'b0),       // 1-bit input: Power-down
  .RST(rst),             // 1-bit input: Reset
  // Feedback Clocks: 1-bit (each) input: Clock feedback ports
  .CLKFBIN(mmcm_clkfb)      // 1-bit input: Feedback clock
);

assign clk_locked = mmcm_locked;

BUFR #(
    .BUFR_DIVIDE("BYPASS"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
    .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_inst (
    .O(clk_1x),     // 1-bit output: Clock output port
    .CE(1'b1),   // 1-bit input: Active high, clock enable (Divided modes only)
    .CLR(1'b0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
    .I(mmcm_clk_1x)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local interconnect
);

BUFIO BUFIO_5x (
    .O(clk_5x), // 1-bit output: Clock output
    .I(mmcm_clk_5x)  // 1-bit input: Clock input
);
assign clk_pix = clk_1x;


wire clk_o;
ODDR #(
    .DDR_CLK_EDGE("OPPOSITE_EDGE"), // "OPPOSITE_EDGE" or "SAME_EDGE" 
    .INIT(1'b0),    // Initial value of Q: 1'b0 or 1'b1
    .SRTYPE("SYNC") // Set/Reset type: "SYNC" or "ASYNC" 
) ODDR_clk (
    .Q(clk_o),
    .C(clk_1x),
    .CE(1'b1),
    .D1(1'b1),
    .D2(1'b0),
    .R(~mmcm_locked),
    .S(1'b0)
);

OBUFDS #(
    .IOSTANDARD("DEFAULT"), // Specify the output I/O standard
    .SLEW("FAST")           // Specify the output slew rate
) OBUFDS_i (
    .O(clk_o_p),     // Diff_p output (connect directly to top-level port)
    .OB(clk_o_n),   // Diff_n output (connect directly to top-level port)
    .I(clk_o)      // Buffer input 
);



wire [7:0] data_in[2:0];
assign data_in[0] = data_in_b;
assign data_in[1] = data_in_g;
assign data_in[2] = data_in_r;

wire [2:0] c0, c1;
assign c0 = {1'b0, 1'b0, hs};
assign c1 = {1'b0, 1'b0, vs};

wire [9:0] data[2:0];
reg [9:0] data_r[2:0];

genvar IB;
generate
for (IB=0; IB < 3; IB=IB+1)
begin: CH
    hdmi_tx_encode encode(
      .clkin(clk_1x),
      .rstin(~mmcm_locked),
      .din(data_in[IB]),
      .c0(c0[IB]),
      .c1(c1[IB]),
      .de(de),
      .dout(data[IB])
    );
    
    always @ (posedge clk_1x)
        data_r[IB] <= data[IB];
    
    hdmi_tx_serdes serdes(
        .pll_locked(mmcm_locked),
        .clk_1x(clk_1x),
        .clk_5x(clk_5x),
        
        .data_in(data_r[IB]),
        .data_o_p(data_o_p[IB]),
        .data_o_n(data_o_n[IB])
    );
end
endgenerate

endmodule
