`timescale 1 ps / 1 ps

module top_level
(
    input        CLK_100M,
    input        RSTBTN,
    
    output       HDMI_OUT_CLK_P,
    output       HDMI_OUT_CLK_N,
    output [2:0] HDMI_OUT_D_P,
    output [2:0] HDMI_OUT_D_N,
    
    output [7:0] LEDS_R,
    output [7:0] LEDS_G,
    output [7:0] LEDS_B
);


wire clk_100;
wire rst;
assign clk_100 = CLK_100M;
assign rst = RSTBTN;


wire clk_pix;

reg [15:0] col;
reg [15:0] row;

parameter HACT    = 1280;
parameter HFP     = 48;
parameter HSYNC   = 112;
parameter HBP     = 248;
parameter HLENGTH = HACT + HFP + HSYNC + HBP;
parameter VACT    = 1024;
parameter VFP     = 1;
parameter VSYNC   = 3;
parameter VBP     = 38;
parameter VLENGTH = VACT + VFP + VSYNC + VBP;

always @ (posedge clk_pix)
if (col==HLENGTH-1)
    col <= 0;
else
    col <= col + 1;
    
    
always @ (posedge clk_pix)
if (col==HLENGTH-1)
    if (row==VLENGTH)
        row <= 0;
    else
        row <= row + 1;

wire de, hs, vs;
wire [7:0] b, g, r;
assign de = (col < HACT) & (row < VACT);
assign hs = (col > (HACT + HFP -1)) & (col < (HACT + HFP + HSYNC));
assign vs = (row > (VACT + VFP -1)) & (row < (VACT + VFP + VSYNC));

reg       de_reg, hs_reg, vs_reg;
reg [7:0] b_reg, g_reg, r_reg;

always @(posedge clk_pix)
begin
   r_reg  <= (col[5] ^ row[5]) ? 8'd255 : 8'd0;
   g_reg  <= (col[5] ^ row[5]) ? 8'd0   : 8'd255;
   b_reg  <= (col[5] ^ row[5]) ? 8'd45  : 8'd200;
   de_reg <= de;
   hs_reg <= hs;
   vs_reg <= vs;
end

wire clk_locked;
hdmi_tx_top hdmi_tx(
    .clk_100(clk_100),
    .rst(rst),
    
    .clk_pix(clk_pix),
    .clk_locked(clk_locked),
    .de(de_reg),
    .hs(hs_reg),
    .vs(vs_reg),
    .data_in_b(b_reg),
    .data_in_g(g_reg),
    .data_in_r(r_reg),
    
    .clk_o_p(HDMI_OUT_CLK_P),
    .clk_o_n(HDMI_OUT_CLK_N),
    .data_o_p(HDMI_OUT_D_P),
    .data_o_n(HDMI_OUT_D_N)
);


assign LEDS_R = {7'b1000000, clk_locked};
assign LEDS_G = 8'b0;
assign LEDS_B = 8'b0;


endmodule
